# -*- perl -*-

# --8<--8<--8<--8<--
#
# Copyright (C) 2006 Smithsonian Astrophysical Observatory
#
# This file is part of mst_envs
#
# mst_envs is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# mst_envs is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#       Free Software Foundation, Inc.
#       51 Franklin Street, Fifth Floor
#       Boston, MA  02110-1301, USA
#
# -->8-->8-->8-->8--

package MST;

use Config;

use strict;
use warnings;

use lib '/proj/axaf/simul/lib/perl5';

use CXC::SysArch;

use SysPathSubst qw( syspathsubst compact_paths );

our @platforms = split( /:/, $CXC::SysArch::Vars{platforms} );

use File::Spec::Functions qw(catdir catfile);

use constant DEFAULT_AXAF_ROOT => '/proj/axaf';
use constant DEFAULT_AXAF_SIMUL_ROOT => '/proj/axaf/simul';

sub split_env
{
    my ( $key ) = @_;

    return exists $ENV{$key} ? split(':', $ENV{$key} ) : ();
}

sub envs
{
  my $opts = shift;
  my %ENV = shift || %ENV;

  my %opts = ( append		=> 0,
	       fullpath		=> 1,
	       ldlibrarypath	=> 0,
	       sysarch		=> 0,
	       $opts ? %$opts : () );

  my $AXAF_ROOT       = $ENV{AXAF_ROOT} || DEFAULT_AXAF_ROOT;
  my $AXAF_SIMUL_ROOT = $ENV{AXAF_SIMUL_ROOT} || DEFAULT_AXAF_SIMUL_ROOT;
  my $AXAF_OTS_ROOT   = catdir( $AXAF_ROOT, 'ots' );

  our %EXPORT;
  $EXPORT{AXAF_ROOT} = $AXAF_ROOT;
  $EXPORT{AXAF_SIMUL_ROOT} = $AXAF_SIMUL_ROOT;
  $EXPORT{AXAF_TEMPLATE_ROOT} = catdir( $AXAF_SIMUL_ROOT, 'lib/templates' );
  $EXPORT{SAOTRACE_DB}        = catdir( $AXAF_SIMUL_ROOT, 'databases' );

  map( $EXPORT{$_} =  $CXC::SysArch::Vars{$_}, keys( %CXC::SysArch::Vars ) )
    if ( $opts{sysarch} );

  our @MANPATH = split_env( 'MANPATH' );
  our @INFOPATH = split_env( 'INFOPATH' );

  if ( $opts{ldlibrarypath} ) {
    my @ld_library_paths;

    push @ld_library_paths, split_env( 'LD_LIBRARY_PATH' )
      if $opts{fullpath} && $opts{append};

    push @ld_library_paths,
      syspathsubst( undef, \@platforms,
		    [ $AXAF_SIMUL_ROOT, '%l/lib' ],
		    [ $AXAF_ROOT      , '%l/lib' ],
		   );

    push @ld_library_paths, split_env( 'LD_LIBRARY_PATH' )
      if $opts{fullpath} && ! $opts{append};

    $EXPORT{LD_LIBRARY_PATH} = compact_paths( ':', @ld_library_paths );
  }

  $EXPORT{MANPATH} =
    syspathsubst( ':', \@platforms, @MANPATH,
		   [ $AXAF_OTS_ROOT  , "pkgs/perl/%l/man" ],
		   [ $AXAF_OTS_ROOT  , "%l/lib/perl/$Config{version}/man" ],
		   [ $AXAF_OTS_ROOT  , "%l/man" ],
		   [ $AXAF_OTS_ROOT  , "man" ],
		   [ $AXAF_OTS_ROOT  , "share/man" ],
		   [ $AXAF_ROOT      , 'man'          ],
		   [ $AXAF_ROOT      , 'share/man'    ],
		   [ $AXAF_SIMUL_ROOT, 'man'          ],
		   [ $AXAF_SIMUL_ROOT, 'share/man'    ],
		   [ $AXAF_SIMUL_ROOT, 'lib/perl/man' ]
		 );


  $EXPORT{INFOPATH} =
    compact_paths( ':', @INFOPATH,
		   [ $AXAF_OTS_ROOT  , "%l/info"	],
		   [ $AXAF_OTS_ROOT  , "info"	],
		   [ $AXAF_OTS_ROOT  , "share/info"	],
		   [ $AXAF_ROOT	     , 'info'		],
		   [ $AXAF_SIMUL_ROOT, 'info'		],
		   [ $AXAF_SIMUL_ROOT, 'share/info'	],
		   );

  my @paths;

  push @paths, $ENV{PATH}
    if $opts{fullpath} && $opts{append};

  push @paths,
    syspathsubst( undef, \@platforms,
		  [ $AXAF_OTS_ROOT  , '%l/bin' ],
		  [ $AXAF_OTS_ROOT  , "pkgs/perl/%l/bin" ],
		  [ $AXAF_OTS_ROOT  , "%l/lib/perl/$Config{version}/bin" ],
		  [ $AXAF_OTS_ROOT  , 'bin'    ],
		  [ $AXAF_ROOT      , 'bin'    ],
		  [ $AXAF_ROOT      , '%l/bin' ],
		  [ $AXAF_SIMUL_ROOT, 'bin'    ],
		  [ $AXAF_SIMUL_ROOT, 'bin/%l' ],
		  [ $AXAF_SIMUL_ROOT, '%l/bin' ],
		 );

  # add Perl binary directory if it isn't one of the standard system
  # ones
  push @paths, $Config{installsitebin}
    if    $Config{installsitebin} ne '/usr/local/bin'
       && $Config{installsitebin} ne '/usr/bin';

  push @paths, $ENV{PATH} || ()
    if $opts{fullpath} && ! $opts{append};

  $EXPORT{PATH} =  compact_paths( ':', @paths );

  # PostgreSQL Setup
  $EXPORT{PGHOST} = 'jeeves';

  # cint Setup
  delete $EXPORT{CINTSYSDIR}
    unless ( $EXPORT{CINTSYSDIR} ) =
      syspathsubst( undef, \@platforms, [ $AXAF_ROOT, "pkgs/cint/%P" ] );

  delete $EXPORT{ROOTSYS}
    unless ( $EXPORT{ROOTSYS} ) =
      syspathsubst( undef, \@platforms, [ $AXAF_ROOT, "pkgs/root/%P"] );


  # Perl Setup
  # convert PERLLIB into PERL5LIB if necessary
  $EXPORT{PERL5LIB} = $ENV{PERLLIB}
    if exists $ENV{PERLLIB} && ! exists $ENV{PERL5LIB};
  $EXPORT{PERL5LIB} =
    syspathsubst( ':', \@platforms,
		  split_env( 'PERL5LIB' ),
		  [ $AXAF_SIMUL_ROOT, 'lib/perl5' ],
		  [ $AXAF_SIMUL_ROOT, '%l/lib/perl5' ],
		);

  # PDL Setup
  $EXPORT{PDLLIB} =
    compact_paths( ':', split_env( 'PDLLIB' ),
		  [ $AXAF_SIMUL_ROOT, 'lib/pdl' ] );

  # Lua Setup

  # figure out which version of Lua is available. capture both STDOUT
  # and STDERR as the message is not written to the same stream by all
  # versions.  do it the old fashioned way to keep dependencies down.
  # don't error out on failure; assume no lua
  eval {
      # Figure out where the lua executable is.  Since the new path
      # hasn't been loaded, if there's one in it we'd better do that
      # one
      my ($luadir ) = grep { -e catfile( $_, 'lua' ) } @paths;

      my $lua = defined $luadir ? catfile( $luadir, 'lua' ) : 'lua';

      open( my $fh, '-|', qq{$lua -v 2>&1} ) or die;
      my $banner = <$fh>;
      close($fh);

      if ( defined($banner) && $banner =~ /Lua\s+(\d+).(\d+)/ )
      {
	  my $version = "$1.$2";

	  my @luadirs =
	    syspathsubst( undef, \@platforms,
			  [ $AXAF_OTS_ROOT,   "%P/lib/lua/$version" ],
			  [ $AXAF_SIMUL_ROOT, "%P/lib/lua/$version" ],
			  [ $AXAF_OTS_ROOT,   "lib/lua/$version" ],
			  [ $AXAF_SIMUL_ROOT, "lib/lua/$version" ],
			);

	  my @lua_paths =
	    map {
		   my $luacdir = $_;
		   map { "$luacdir/$_" } qw( ?.lc ?.lua ?/init.lc ?/init.lua )
	       } @luadirs;

          $EXPORT{LUA_PATH} = join(';', @lua_paths);

	  $EXPORT{LUA_PATH} .= exists $ENV{LUA_PATH}
	                     ? ';' . $ENV{LUA_PATH}
	                     : ';;';
      }
  };

  # PkgConfig Setup
  $EXPORT{PKG_CONFIG_PATH} =
    syspathsubst( ':', \@platforms,
		  split_env( 'PKG_CONFIG_PATH' ),
		  [ $AXAF_OTS_ROOT  , "lib/pkgconfig"    ],
		  [ $AXAF_OTS_ROOT  , "%l/lib/pkgconfig" ],
		  [ $AXAF_SIMUL_ROOT, "lib/pkgconfig"    ],
		  [ $AXAF_SIMUL_ROOT, "%l/lib/pkgconfig" ],
		  [ $AXAF_ROOT      , "lib/pkgconfig"    ],
		  [ $AXAF_ROOT      , "%l/lib/pkgconfig" ],
		  '/usr/lib64/pkgconfig',
		  '/usr/lib/pkgconfig',
		  '/usr/share/pkgconfig',
		);

  # locate database setup.
  $EXPORT{MST_LOCATE_PATH} =
    compact_paths( ':', glob("$AXAF_ROOT/var/lib/mlocate/*.db") );

  # CVS Repository
  $EXPORT{CVSROOT} = catdir( $AXAF_SIMUL_ROOT, 'src/cvs' )
    unless exists $ENV{CVSROOT};

  # Parameter File default directories
  my ( $w_dirs, $ro_dirs );

  ( $w_dirs, $ro_dirs ) = $ENV{PFILES} =~ /([^;]*)(?:|;(.*))$/
    if exists $ENV{PFILES};

  $w_dirs ||= '';
  $ro_dirs ||= '';

  $w_dirs = compact_paths( ':',
			   # must use './' to work around bug in cxcparam library
			   './',
			   './uparm',
			   [ $ENV{HOME}, 'mst_param' ],
			   split(':', $w_dirs),
			 );
  $ro_dirs = compact_paths( ':',
			    split(':',$ro_dirs),
			    [ $AXAF_SIMUL_ROOT, 'share/uparm' ],
			    [ $AXAF_SIMUL_ROOT, 'lib/uparm'   ],
			  );
  $EXPORT{PFILES} = $w_dirs . ';' . $ro_dirs;


  # TEXINPUTS
  #  this has to be handled differently as an empty path component is
  #  valid.  currently syspathsubst checks for the existence of a
  #  path, and an empty path will get removed.  So, only use
  #  syspathsubst on the paths specified here.

  my $TEXINPUTS =
    syspathsubst( ':', \@platforms,
                  [ $AXAF_SIMUL_ROOT, "%l/share/texmf" ],
                  [ $AXAF_SIMUL_ROOT, "share/texmf"    ],
                  [ $AXAF_OTS_ROOT  , "%l/share/texmf" ],
                  [ $AXAF_OTS_ROOT  , 'share/texmf'    ],
                  [ $AXAF_ROOT      , "%l/share/texmf" ],
                  [ $AXAF_ROOT      , 'share/texmf'    ],
                );

  # make sure to add // to each to get text to descend into the directory
  my @TEXINPUTS = map { $_ . '//' } split( ':', $TEXINPUTS );

  $EXPORT{TEXINPUTS} =
    join(':',
         @TEXINPUTS,

         # if the user doesn't have TEXINPUTS set, then setting it without
         # including the default path will mess up the user. the empty
         # path here causes a trailing ':' to be appended which is
         # kpsearch's way of marking where the default TEXINPUTS value
         # should be inserted.
         exists $ENV{TEXINPUTS} ? $ENV{TEXINPUTS} : '' ,
        );


  delete $EXPORT{$_} foreach grep { ! defined $EXPORT{$_}
				    || $EXPORT{$_} eq '' } keys %EXPORT;

  %EXPORT;
}

1;
